/*
 * @start_prolog@
 * ============================================================================
 *  Licensed Materials - Property of IBM
 *  
 *  5724-J08, 5724-I63, 5724-H88, 5724-H89, 5655-N02, 5733-W70
 *  
 *  Copyright IBM Corp. 2009
 * ============================================================================
 * @end_prolog@
 * 
 * Change activity:
 * 
 * Issue       Date        Name           Description
 * ----------- ----------- -------------- ------------------------------------
 */
package com.ibm.ws.eba.example.blabber.persistence.spi;

import java.util.List;
import java.util.Set;

import com.ibm.ws.eba.example.blabber.persistence.entity.Post;
import com.ibm.ws.eba.example.blabber.persistence.entity.UserInfo;

public interface BlabberUserInterface
{
	/**
	 * Persist a user's Post into the database
	 * @param username - the user
	 * @param content - the status
	 */
	void postContent(String username, String content);
	
	/**
	 * 	Get all the Posts created by the user, username
	 *  @param username - the user of interest
	 *  @return a List of Posts by the user
	 */
	List<Post> getPostsByUser(String username);
	
	/**
	 * Get all the Posts created by the user, and by other users that the supplied user is following
	 * @param username - the user of interest
	 * @return a List of Posts to display on the user profile
	 */
	List<Post> getPostsForUserProfile(String username);
	
	/**
	 * Get all the Posts by users tracked by user, username, that mention user, username
	 * @param username - the user of interest
	 * @return a List of Posts
	 */
	List<Post> getMentionsOfUser(String username);
	
	/**
	 * Request to track the Posts of another user
	 * @param userThatWillFollow - the user that is requesting to track another user
	 * @param userToBeFollowed - the user that is to be tracked
	 * @return true if the operation is successful and false otherwise
	 */
	boolean startTracking(String userThatWillFollow, String userToBeFollowed);
	
    /**
     * Request to stop tracking the Posts of someone else
     * @param userThatWillFollow - the user that is tracking someone else
     * @param userToBeFollowed - the user that is no longer to be tracked
     * @return true if the operation is successful and false otherwise
     */
	boolean stopTracking(String userThatWillFollow, String userToBeFollowed);

	/**
	 * Registers a new user
	 * @param username - the username
	 * @param password - the user's password
	 * @param emailAddress - the user's email address
	 * @param avatar - the integer that represents the user's chosen avatar
	 * @return true if registration is successful and false otherwise
	 */
	boolean registerUser(String username, String password, String emailAddress, int avatar);
	
	/**
	 * Logon an existing user
	 * @param username - the user logging in
	 * @param password - the user's supplied password
	 * @return true if credentials match and false otherwise
	 */
	boolean logon(String username, String password);

	/**
	 * Checks whether the supplied username is already in use
	 * @param userName - the username to check
	 * @return true if the username has not yet been registered, false otherwise
	 */
	boolean checkUsernameUniqueness(String userName);
	
	/**
	 * Checks whether the supplied e-mail address has been registered against a username
	 * @param emailAddress - the e-mail address to check 
	 * @return true if the e-mail address has NOT been registered against a username, false otherwise
	 */
	boolean checkEMailAddressUniqueness(String emailAddress);

	/**
	 * Searches for usernames that match the supplied filter
	 * @param userNameFilter - a full or partial username
	 * @return a List of users that match the filter
	 */
	List<UserInfo> searchUsername(String usernameFilter);
	
	/**
	 * Checks whether one user is tracking another
	 * @param tracker - the user who is tracking
	 * @param tracked - the user who is potentially being tracked
	 * @return true if tracking, false otherwise
	 */
	boolean isTracking(String tracker, String tracked);
	
	/**
	 * Gets all the users that are tracked by the user, username
	 * @param username - the user of interest
	 * @return a Set of users that are being tracked
	 */
	Set<UserInfo> getTrackedUsers(String username);
	
	/**
	 * Gets the avatar for the requested user
	 * @param username - the user of interest
	 * @return an integer representing the avatar associated with the user
	 */
	int getUserAvatar(String username);
}
